Init()
{
	if(getCvar("scr_allow_medic") == "")
		setCvar("scr_allow_medic", 0);
	level.allow_medic = getCvarInt("scr_allow_medic");

	level.medic_distance = 90;

	thread onPlayerConnect();
}

Precache()
{
	game["hud_healing"] = &"AA_MD_MEDIC_02";
	precacheString(game["hud_healing"]);

	game["hud_healer"] = &"AA_MD_MEDIC_03";
	precacheString(game["hud_healer"]);

	game["hud_heal_attempt"] = &"AA_MD_MEDIC_01";
	precacheString(game["hud_heal_attempt"]);
}

onPlayerConnect()
{
	if (!level.allow_medic) return;

	for(;;)
	{
		level waittill("connecting", player);

		player thread onPlayerSpawned();
		player thread onPlayerKilled();
	}
}

onPlayerSpawned()
{
	self endon("disconnect");
	
	for(;;)
	{
		self waittill("spawned_player");
		self.bomb_interraction = 0;
		self.healing = 0;
		self.beinghealed = 0;
		self.attempting_heal = 0;
		self.attempting_being_heal = 0;

		self notify("update_hud");

		self.cross_hud = undefined;
		self.healer = undefined;
		
		self thread Medic_Scan();
		self thread HUD_Scan();
	}
}

onPlayerKilled()
{
	self endon("disconnect");
	
	for(;;)
	{
		self waittill("killed_player");
		self.bomb_interraction = 0;
		self.healing = 0;
		self.beinghealed = 0;
		self.attempting_heal = 0;
		self.attempting_being_heal = 0;

		self notify("update_hud");
	}
}

Medic_Scan()
{
	self endon("disconnect");
	self endon("player_killed");

	wounded = self;
	old_wounded = self;
	hud_change = 0;

	while (1)
	{
		wait .05;
		delay = 0;
		self.attempting_heal = 0;
		if (hud_change)
		{
			self notify("update_hud");
			hud_change = 0;
		}

		while ( !self.healing && !self.beinghealed && !self.bomb_interraction )
		{
			wait .1;

			if ( !self useButtonPressed() || self.attempting_being_heal)
			{
				if (self.attempting_heal)
					hud_change = 1;
				break;
			}

			wounded = self Check_Eligible_Targets();
			if (wounded != old_wounded || wounded == self)
			{
				if (self.attempting_heal)
					hud_change = 1;
				old_wounded = wounded;
				break;
			}

			if (delay == 6)
			{
				self.attempting_heal = 1;
				self notify("update_hud");
				hud_change = 1;
				self thread Attempting_Being_Healed(wounded);
			}

			if (delay < 16)
			{
				delay++;
				continue;
			}

			self thread Setup_Actors(Wounded);		
		}
	}
}

Check_Eligible_Targets()
{
	Eligible_Player = self;

	players = getentarray("player", "classname");
	for(i = 0; i < players.size; i++)
	{
		player = players[i];

		if (player == self) continue;
		if (player.health == 100 || !isAlive(player) || player.pers["team"] != self.pers["team"] || player.bomb_interraction) continue;
		if ( distance(player.origin, self.origin) > level.medic_distance) continue;

		Eligible_Player = player;

		//To Do: Maybe scan ALL players and select the CLOSEST eligible player?
		break;
	}

	return Eligible_Player;
}

Setup_Actors(wounded)
{
	self.sitstill = spawn("script_origin", self.origin);
	self linkTo(self.sitstill);
	self disableWeapon();

	wounded.sitstill = spawn("script_origin", wounded.origin);
	wounded linkTo(wounded.sitstill);
	wounded disableWeapon();

	//HUD Elements
	self.attempting_heal = 0;
	wounded.attempting_being_heal = 0;
	self notify("update_hud");
	wounded notify("update_hud");
	wait .05; // Time to Clear Old HUDs

	self.healing = 1;
	wounded.beinghealed = 1;
	self notify("update_hud");
	wounded notify("update_hud");

	self Heal(wounded);

	if (isDefined(self) && isAlive(self))
	{
		self unlink();
		self enableWeapon();
		self.healing = 0;

		self notify("update_hud");
	}

	if (isDefined(wounded) && isAlive(wounded))
	{
		wounded unlink();
		wounded enableWeapon();
		wounded.beinghealed = 0;

		wounded notify("update_hud");
	}

	if (isDefined(self.sitstill))
		self.sitstill delete();
	if (isDefined(wounded.sitstill))
		wounded.sitstill delete();
}

Heal(wounded)
{
	timer = 0;
	heal_time = 2;
	heal_amount = 1;

	self thread Heal_Sounds(wounded);

	wounded.bleeding = 0;

	while (isDefined(self) && isDefined(wounded) && self useButtonPressed() && wounded.health < 100)
	{
		if ( distance(self.origin, wounded.origin) > level.medic_distance) break;

		if (!self useButtonPressed() 
		|| !isAlive(self) 
		|| !isAlive(wounded) 
		|| self.sessionstate != "playing" 
		|| wounded.sessionstate != "playing")
		{
			break;
		}
		
		if (timer == heal_time)
		{
			wounded.health = wounded.health + heal_amount;
			if (wounded.health > 100)
				wounded.health = 100;

			timer = -1;
		}

		wait .05;
		timer++;
	}
}

Heal_Sounds(wounded)
{
	wait .75;
	self playLocalSound("health_pickup_small");
	wounded playLocalSound("health_pickup_small");

	num = -1;
	delay = 0;
	timer = 0;

	while ( isDefined(wounded) && wounded.beinghealed )
	{
		if (num == -1)
		{
			num = randomint(2);
			delay = num + 2;
			timer = 0;
		}

		if (timer >= delay)
		{
			self playLocalSound("health_pickup_small");
			wounded playLocalSound("health_pickup_small");
			wait .2;
			self playLocalSound("breathing_hurt");
			wounded playLocalSound("breathing_hurt");
			num = -1;
		}

		timer = timer + .5;

		wait .5;
	}

	if ( isDefined(self) && isAlive(self) )
		self playLocalSound("breathing_better");
	if ( isDefined(wounded) && isAlive(wounded) )
		wounded playLocalSound("breathing_better");
}

HUD_Scan()
{
	self endon("disconnect");
	self endon("player_killed");

	while (1)
	{
		self waittill("update_hud");
		wait .05;

		if ( (self.healing || self.beinghealed) && !isDefined(self.cross_hud) )
		{
			self.cross_hud = newClientHudElem(self);
			self.cross_hud.x = 580;
			self.cross_hud.y = 100;
			self.cross_hud.alignX = "center";
			self.cross_hud.alignY = "middle";
			self.cross_hud.alpha = 1;
			self.cross_hud setShader(game["health_cross"], 24, 24);
		}
		else if ( !self.healing && !self.beinghealed && isDefined(self.cross_hud))
			self.cross_hud destroy();

		if (self.attempting_heal && !isDefined(self.cross_hud) )
		{
			self.heal_hud_attempt = newClientHudElem(self);
			self.heal_hud_attempt.x = 580;
			self.heal_hud_attempt.y = 130;
			self.heal_hud_attempt.alignX = "center";
			self.heal_hud_attempt.alignY = "middle";
			self.heal_hud_attempt.fontScale = 1.2;
			self.heal_hud_attempt.color = (1, 1, 1);
			self.heal_hud_attempt.alpha = 1;
			self.heal_hud_attempt setText(game["hud_heal_attempt"]);
		}
		else if (!self.attempting_heal && isDefined(self.heal_hud_attempt))
			self.heal_hud_attempt destroy();

		if ( (self.healing || self.beinghealed) && !isDefined(self.heal_hud) )
		{
			self.heal_hud = newClientHudElem(self);
			self.heal_hud.x = 580;
			self.heal_hud.y = 130;
			self.heal_hud.alignX = "center";
			self.heal_hud.alignY = "middle";
			self.heal_hud.fontScale = 1.2;
			self.heal_hud.color = (1, 1, 1);
			self.heal_hud.alpha = 1;

			if (self.healing)
				self.heal_hud setText(game["hud_healer"]);
			else
				self.heal_hud setText(game["hud_healing"]);

		}
		else if ( !self.healing && !self.beinghealed && isDefined(self.heal_hud))
			self.heal_hud destroy();
	}

}

Attempting_Being_Healed(wounded)
{
	wounded.attempting_being_heal = 1;
	wounded notify("update_hud");

	while (1)
	{
		wait .15;

		if ( !isDefined (self) || !isDefined(wounded) || !isAlive(self) || wounded.beinghealed)
		{
			if (isDefined(wounded))
			{
				wounded.attempting_being_heal = 0;
				wounded notify("update_hud");
			}

			return;
		}
	}
}