Init()
{
	level.mortars = 0;
	if (getcvar("scr_allow_mortars") != "" && getcvarint("scr_allow_mortars") > 0)
		level.mortars = getcvarint("scr_allow_mortars");

	if (level.mortars)
		Setup_Mortars();
}

Setup_Mortars()
{
	if(!isdefined(game["gamestarted"]))
	{
		// mortar model
		game["mortar_model"] = "xmodel/vehicle_halftrack_rockets_shell";
		precacheModel(game["mortar_model"]);

		// mortar warning sounds
		game["incoming0"] = "mortar_incoming2";
		game["delay0"] = 0.65;
		game["incoming1"] = "mortar_incoming1";
		game["delay1"] = 1.05;
		game["incoming2"] = "mortar_incoming3";
		game["delay2"] = 1.5;
		game["incoming3"] = "mortar_incoming4";
		game["delay3"] = 2.1;
		game["incoming4"] = "mortar_incoming5";
		game["delay4"] = 3.0;

		// load effects
		game["mortarfx_generic"] = loadfx("fx/explosions/mortarExp_mud.efx");
		game["mortarfx_sand"] = loadfx("fx/explosions/mortarExp_beach.efx");
		game["mortarfx_snow"] = loadfx("fx/explosions/large_snow_explode1.efx");
		game["mortarfx_water"] = loadfx("fx/explosions/mortarExp_water.efx");
		game["mortarfx_concrete"] = loadfx("fx/explosions/mortarExp_concrete.efx");
		game["mortarfx_dirt"] = loadfx("fx/explosions/mortarExp_dirt.efx");
		game["mortarfx_grass"] = loadfx("fx/explosions/artilleryExp_grass.efx");
	}

	level.safe_mortars = 0;
	if (getcvar("scr_mortar_safety") != "" && getcvarint("scr_mortar_safety") > 0)
		level.safe_mortars = 1;

	level.mortar_quake = 0;
	if (getcvar("scr_mortar_quake") != "" && getcvarint("scr_mortar_quake") > 0)
		level.mortar_quake = 1;

	level.mortar_delay_min = 15;
	if (getcvar("scr_mortar_reload_min") != "" && getcvarint("scr_mortar_reload_min") > 0)
		level.mortar_delay_min = getcvarint("scr_mortar_reload_min");
	if (level.mortar_delay_min > 120)
		level.mortar_delay_min = 120;
	if (level.mortar_delay_min < 5)
		level.mortar_delay_min = 5;

	level.mortar_delay_max = 30;
	if (getcvar("scr_mortar_reload_max") != "" && getcvarint("scr_mortar_reload_max") > 0)
		level.mortar_delay_max = getcvarint("scr_mortar_reload_max");
	if (level.mortar_delay_max > 240)
		level.mortar_delay_max = 240;
	if (level.mortar_delay_max < level.mortar_delay_min)
		level.mortar_delay_max = level.mortar_delay_min+1;

	mortar_guns = 2;
	if (getcvar("scr_mortar_guns") != "" && getcvarint("scr_mortar_guns") > 0)
		mortar_guns = getcvarint("scr_mortar_guns");
	if (mortar_guns > 15)
		mortar_guns = 15;

	for ( i=0 ; i<mortar_guns ; i++ )
		thread Mortar_Gun_Think();
}

Mortar_Gun_Think()
{
	level endon("round_ended");

	maxz = game["map_area_max"][2];	
	delay_range = int(level.mortar_delay_max - level.mortar_delay_min);

	// Init some local variables
	endorigin = (0,0,0);
	m = 0;
	pc = 0;
	x = 0;
	y = 0;
	trace = [];
	surface = "generic";

	// Do an endless loop
	for(;;)
	{
		if (isDefined(level.mapended) && level.mapended)
			return;

		// Get a random delay
		delay = randomInt(delay_range);
		delay = level.mortar_delay_min + delay;

		wait delay;

		// Spawn mortar model
		mortar = spawn("script_model", (0,0,0));
		mortar setModel(game["mortar_model"]);
		mortar hide();	// Hide it until we need it


		// Get a random mortar incoming sound
		sound_num = randomInt(5);
		sound = game["incoming"+sound_num];
		sound_delay = game["delay"+sound_num];

		// Random strength
		pc = randomInt(100);

		// Get it's damage range
		range = int(200 + pc*3.6);


		// Loop until we find a valid spot
		if (level.safe_mortars)
		{
			min_distance = range * 1.25;
			max_distance = min_distance + 900; // (600 = 50 feet)
		}
		else
		{
			min_distance = range * .9;
			max_distance = min_distance + 1100; // (600 = 50 feet)
		}

		// Loop until we find a valid spot
		acceptable = 0;
		while(!acceptable)
		{
			//get random ending location
			x = game["play_area_min"][0] + randomInt(int(game["play_area_max"][0]-game["play_area_min"][0]));
			y = game["play_area_min"][1] + randomInt(int(game["play_area_max"][1]-game["play_area_min"][1]));
			z = game["play_area_min"][2];
			endorigin = (x,y,z);

			// Check if players are too close (safety) or too far
			acceptable = 1;
			players = getentarray("player", "classname");
			for(i = 0; i < players.size; i++)
			{
				player = players[i];

				if (player.sessionstate != "playing")
					continue;

				distance = distance(endorigin, player.origin);

				if (distance < min_distance || distance > max_distance)
				{
					acceptable = 0;
					break;
				}
			}

			if (acceptable)
			{
				// find the impact point
				trace = bulletTrace((endorigin[0],endorigin[1],maxz),(endorigin[0],endorigin[1],game["map_area_min"][2]), false, undefined);
				endorigin = trace["position"];
			}
			else
				wait .2;
		}

		// Get surfacetype
		get_surface = trace["surfacetype"];

		// Start point for mortar
		startpoint = ( (endorigin[0] - 200 + randomInt(400)) , (endorigin[1] - 200 + randomInt(400)) ,maxz);
		mortar.origin = startpoint;
		wait .05;

		// play the incoming sound
		mortar playsound(sound);

		// wait for it to hit
		falltime = .5;
		wait sound_delay - 0.05 - falltime;

		// Show visible mortar object
		mortar.angles = vectortoangles(vectornormalize(mortar.origin - startpoint));
		mortar show();
		wait .05;

		// Move visible mortar
		mortar moveto(endorigin, falltime);

		// wait for it to hit
		wait falltime;

		// play the visual effect if we have a suitable loaded
		surf_effect = game["mortarfx_"+trace["surfacetype"]];
		//iprintln("Surface:" + trace["surfacetype"]);
		if(!isdefined(surf_effect))
		{
			if (trace["surfacetype"] == "rock")
				surf_effect = game["mortarfx_concrete"];
			else if (trace["surfacetype"] == "default" || trace["surfacetype"] == "wood")
				surf_effect = game["mortarfx_generic"];
			else
			{
				surf_effect = game["mortarfx_generic"];
				//iprintln("Unknown Existing surface:" + trace["surfacetype"]);
			}
		}

		playfx(surf_effect, endorigin);

		// Play explosion sound
		mortar playsound("mortar_explosion" + (randomInt(5) + 1));

		// Hide visible mortar
		mortar hide();

		// just to be extra safe, since a player MIGHT move in range during the "incoming" sound
		if(!level.safe_mortars)
		{
			// do the damage
			max = 200 + 2*pc;
			min = 10;
			radiusDamage(endorigin + (0,0,12), range, max, min);
		}

		if(level.mortar_quake)
		{
			// rock their world
			strength = 0.5 + 0.005 * pc;
			length = 1 + pc*0.03;
			range = int(600 + 6*pc);
			earthquake(strength, length, endorigin, range); 
		}

		mortar delete();
	}
}